% examples of Chapter 1
%
% G. Doblinger, TU-Wien, 10-2009

n = -10:10;                   % create a row vector n containing integers
                              % (time points) -10, -9 , ... 9, 10
x = 0.8 * sin(2*pi/20*n);     % create a sinusoidal signal stored in
                              % column vector x
stem(n,x);                    % discrete-time plot
xlabel('n'), ylabel('x[n]');

%%
n = -10:10;                   % create a row vector n containing integers
                              % (time points) -10, -9 , ... 9, 10
f = 6000;                     % frequency in Hz
Fs = 8000;                    % sampling frequency in Hz
x = sin(2*pi*f/Fs*n);
stem(n,x);
xlabel('n'), ylabel('x[n]');

% show aliasing when sampling an continuous-time signal

%%
N = 40;                       % number of samples
m = 7;
f = m/N*Fs;
fprintf(1,'f = %4.2f Hz\n',f);
dtcos(N,m);                   % no aliasing, f < Fs/2

%%
m = 17;
f = m/N*Fs;
fprintf(1,'f = %4.2f Hz\n',f);
dtcos(N,m);                   % no aliasing, f < Fs/2

%%
m = 27;
f = m/N*Fs;
fprintf(1,'f = %4.2f Hz\n',f);
dtcos(N,m);                   % aliasing, f > Fs/2

%%

close all
nx = 20;
x1 = zeros(1,nx);             % create a row vector of 20 zeros
i1 = 1:2:nx;                  % row vector of odd integers (1,3,5,...19)
x1(i1) = 1;                   % set every other element of x1 to one
stem(x1);                     % plot x1
xlabel('n'), ylabel('x[n]');

%%
n = -20:20;
x = sin(0.1*pi*n);
x1 = x;
i1 = find(x1 < 0);            % find indices of x1 where vector
                              % are negative
x1(i1) = 0;                   % replace negative signal parts by zeros
subplot(2,1,1),stem(n,x);
xlabel('n'), ylabel('x[n]');
subplot(2,1,2),stem(n,x1);
xlabel('n'), ylabel('x_1[n]');

%%
x2 = 2*x;
Aclip = 0.5;
i2p = find(x2 > Aclip);
i2n = find(x2 < -Aclip);
x2(i2p) = Aclip;              % clip positive half waves
x2(i2n) = -Aclip;             % clip negative half waves
subplot(2,1,1),plot([n(1) n(end)],[Aclip Aclip],'r--',...
   [n(1) n(end)],[-Aclip -Aclip],'r--');
hold on
subplot(2,1,1),stem(n,2*x);
hold off
xlabel('n'), ylabel('x[n]');
subplot(2,1,2),stem(n,x2);
xlabel('n'), ylabel('x_2[n]');

%%
close all
x = sin(0.1*pi*n);
x1 = (x > 0);                 % x1 is set to 1 where x > 0, and x1 = 0
                              % elsewhere, i.e. a square wave is created
                              % from the original sinusoid
subplot(2,1,1),stem(n,x);
xlabel('n'), ylabel('x[n]');
subplot(2,1,2),stem(n,x1);
xlabel('n'), ylabel('x_1[n]');

%%

close all
n = -20:20;
N = 5;
x = (mod(n,N) == 0);          % x2 = 1 at multiples of N, i.e. a unit
                              % pulse train with period N is generated
                              % (mod() is the modulo operation)
stem(n,x);
xlabel('n'), ylabel('x[n]');

%%
n = 0:20;                     % vector of time points 0, 1, 2 ... 20
alpha = 0.8;
x = alpha .^ n;               % operator .^ performs a componentwise
                              % exponentiation
stem(n,x);
xlabel('n'), ylabel('x[n]');

%%

% elementwise operations on vectors

clc
x = [1 ; 2 ; 3 ; 4 ; 5];      % 5 x 1 column vector
disp('x = '), disp(x)
y = x.*x;                     % elementwise product (Hadamard product)
disp('y = x.*x'),disp(y)
y = x'*x;                     % scalar (inner) vector product
disp('y = x''*x'),disp(y)
y = x*x';                     % outer vector product
disp('y = x*x'''),disp(y)

%%
% special case: repeat columns

x = [1 ; 2 ; 3 ; 4 ; 5];      % 5 x 1 column vector
y = x*ones(1,4);
disp('y = x*ones(1,4)'),disp(y)

% compact form avoiding multiplications by 1

y = x(:,ones(1,4));
disp('y = x(:,ones(1,4))'),disp(y)

%%
% special case: repeat rows

x = [1  2  3  4  5];      % 1 x 5 row vector
y = ones(4,1)*x;
disp('y = ones(4,1)*x'),disp(y)

% compact form avoiding multiplications by 1

y = x(ones(4,1),:);
disp('y = x(ones(4,1),:)'),disp(y)

%%

% elementwise operations on matrices

clc
A = [1 2 ; 3 4];
disp('A = '), disp(A)
B = A^2;                      % matrix multiplication AA 
disp('B = A^2 = AA'), disp(B)
C = A.^2;                     % Hadamard product A o A
disp('C = A.^2 = AoA'), disp(C)

%%

% reshaping matrices to vectors and vice versa

A = [ 1 2 3 ; 4 5 6];
disp('A = '), disp(A)
a = A(:);                     % a = vec(A), stacking all columns to a vector
disp('a = A(:)'), disp(a)
B = zeros(3,2);
B(:) = a;                     % reshaping vector to matrix 
disp('B(:) = a'), disp(B)

%%

% example solving a set of linear equations

A = randn(3);
disp('A = '), disp(A)
b = randn(3,1);
disp('b = '),disp(b) 
x = A\b;                      % solution of Ax = b
disp('x = A\b'), disp(x)

%%

% does this work if b is a matrix, i.e. the solution is also a matrix?

clc
A = randn(3);
disp('A = '), disp(A)
B = randn(3);
disp('B = '), disp(B)
X = A\B;                      % solution of AX = B
disp('X = A\B'), disp(X)

% solution using vec-operation and Kronecker product
% AX = B  -> A[x1 x2 x3] = [b1 b2 b3] -> Ax1 = b1, Ax2 = b2, Ax3 = b3
% -> [A 0 0 ; 0 A 0 ; 0 0 A][x1 ; x2 ; x3] = [b1 ; b2 ; b3]
% more compact: kron(I,A)vec(X) = vec(B), I = identity matrix

x = kron(eye(3),A)\B(:);
disp('x = kron(eye(3),A)\B(:)'), disp(x)
X = zeros(3);
X(:) = x;
disp('X(:) = x'), disp(X)

%%

% time shift (positive delay)

n = 0:20;
x = 0.8 .^ n;                 % create exponentially decaying signal
Nd = 10;                      % time delay in samples
y = [zeros(1,Nd-1),x];        % add Nd-1 leading zeros to delay signal
subplot(2,1,1),stem(n,x);
xlabel('n'), ylabel('x[n]');
subplot(2,1,2),stem(n,y);
xlabel('n'), ylabel('y[n]');

%%

% how to implement a negative delay (advance)?

x = 0.8 .^ (0:20);            % create a row vector of exponentially
                              % decaying signal samples
x = [zeros(1,20) x];          % add leading zeros to vector x to delay
                              % signal in order to use negative delays
Nd = 10;                              
x1 = x(Nd+1:end);             % shift signal to the left
x2 = [zeros(1,Nd-1) x];       % shift singal to the  right  
n = 0:length(x)-1;
subplot(3,1,1),stem(n,x);
xlabel('n'), ylabel('x[n]'), title(['Nd = ',int2str(Nd)]);
axis([0 length(x)+Nd -inf inf])
n = 0:length(x1)-1;
subplot(3,1,2),stem(n,x1);
xlabel('n'), ylabel('x[n+Nd]');
axis([0 length(x)+Nd -inf inf])
n = 0:length(x2)-1;
subplot(3,1,3),stem(n,x2);
xlabel('n'), ylabel('x[n-Nd]');
axis([0 length(x)+Nd -inf inf])

%%

% time reversal

x_rev = x(end:-1:1);          % perform time-inversion by reversing
                              % order of vector components
                              % Note: end = index of last x-component 
n = 0:length(x)-1;
subplot(2,1,1),stem(n,x);
xlabel('n'), ylabel('x[n]')
subplot(2,1,2),stem(n,x_rev);
xlabel('n'), ylabel('x[-n]')

%%

% creating periodic signals by repeating the fundamental signal part

N = 20;                      % period
n = 0:N-1;
x = 0.7 .^ n;                % finite length signal (fundamental signal part)
np = 0:4*N-1;                % time indices of periodic signal  
xp = x(mod(np,N)+1);         % use modulo operation rem() to periodically
                             % extend signal x
subplot(2,1,1),stem(n,x);
xlabel('n'), ylabel('x[n]')
axis([0 length(xp) -inf inf])
subplot(2,1,2),stem(np,xp);
xlabel('n'), ylabel('x_p[n]')

%%

% cyclic shift

N = 20;                      % period
Nd = 10;                     % time-shift in samples 
n = 0:N-1;
x = 0.7 .^ n;                % finite length signal
np = 0:4*N-1;
xp = x(mod(np,N)+1);
xp1 = x(mod(np+Nd,N)+1);     % cyclic time-shift by Nd samples
subplot(2,1,1),stem(np,xp);
xlabel('n'), ylabel('x[n]')
subplot(2,1,2),stem(np,xp1);
xlabel('n'), ylabel('x[n-Nd]'), title(['Nd = ',int2str(Nd)]);

%%

% decimation of speech signal

x = wavread('male_1ch_16kHz.wav');
Nx = 10000;                  % Nx samples of x will be used 
M = 40;                      % decimation factor
Nb = floor(Nx/M);
Nx = M*Nb;                   % Nx is a multiple of M
X = zeros(M,Nb);
X(:) = x(1:Nx);              % columns of X contaim frames of length M
xmax = max(X);               % maxima of all columns of X
xmin = min(X);

close all
t = 0:Nx-1;
subplot(3,1,1), plot(t,x(1:Nx)), grid on;
xlabel('n'), ylabel('x[n]');
tb = 0:Nb-1;
subplot(3,1,2), plot(tb,x(1:M:Nx)), grid on;      % plot every Mth sample
xlabel('n (decimated)'), ylabel('x_d[n]');
title(sprintf('M = %d',M));
subplot(3,1,3),
hold on, area(tb,xmax), area(tb,xmin), hold off;  % plot max/min of frames
colormap([0 0.6 1]), grid on;
xlabel('n (decimated)'), ylabel('x_e[n]');

%%

% matrix representation of convolution

close all;

Nx = 4;
Nh = 3;
x = ones(Nx,1);              % rectangular signal 
h = ones(Nh,1);              % rectangular filter impulse response 
Ny = Nx+Nh-1;                % length of convolution result
H = zeros(Ny,Nx);
for n = 1:Nx                 % build matrix
   H(n:n+Nh-1,n) = h;
end
y = H*x;                     % perform convolution
stem((0:Ny-1),y);
xlabel('n'), ylabel('y[n]');

%%

% alternative without for loop (looks very strange)

h1 = [zeros(Nx-1,1,1) ; h ; zeros(Nx-1,1)];  % add zeros to vector h
ic = (0:Ny-1)';
ir = Nx:-1:1;
H = ic(:,ones(Nx,1)) + ir(ones(Ny,1),:);     % matrix of indices where h1
                                             % is to be placed
H(:) = h1(H);

y = H*x;                     % perform convolution
stem((0:Ny-1),y);
xlabel('n'), ylabel('y[n]');

%%

% correlation function of a sinusoidal chirp

Nx = 300;
n = 0:Nx-1;
x = sin(pi/3000*n.^2);
y = 1/Nx*conv(x,x(end:-1:1));
subplot(2,1,1),plot(n,x), grid on
xlabel('time index n'), ylabel('x[n]');
subplot(2,1,2),plot(y), grid on
xlabel('lag index k'), ylabel('R_{xx}[k]');

%%

% correlation function of Gaussian white noise

xn = randn(size(x));
yn = 1/Nx*conv(xn,xn(end:-1:1));
subplot(2,1,1),plot(n,xn), grid on
xlabel('time index n'), ylabel('x[n]');
subplot(2,1,2),plot(yn),grid on
xlabel('lag index k'), ylabel('R_{xx}[k]');

