function D = BestBasisTree(D)
%function D = BestBasisTree(D)
%
% find best basis tree for full binary wavelet packet tree
% D            structure of node data
%              D.coef     node coefficients concatenated to row vector
%              D.start    start index of coeff. for each node
%              D.stop     stop index of coeff. for each node
%              D.root     index of root node for each node (top node = index 0)
%              D.entr     entropy for each node
%              D.engy     energy for each node

% G. Doblinger, TU-Wien, 7.12.97

% nodes to be eliminated are marked with entropy D.entr = NaN
% Note: topmost node (index 0) is not used)

Nnodes = length(D.entr);   % number od nodes in the tree

% mark nodes starting from second lowest level of tree

for n = Nnodes-1:-2:3
   if ~isnan(D.entr(n))
      entr_leaves = D.entr(n) + D.entr(n+1);
      if entr_leaves > D.entr(D.root(n))
         D.entr(n) = NaN;                    % mark leaves, if they contribute more cost
         D.entr(n+1) = NaN;
      else
         D.entr(D.root(n)) = entr_leaves;    % change root entropy
      end
   end
end

% elemininate possible stand-alone nodes

for n = 3:2:Nnodes-1
   if isnan(D.entr((D.root(n))))
      D.entr(n) = NaN;
      D.entr(n+1) = NaN;
   end
end
