function [tn,an] = TermNodes(D)
%function [tn,an] = TermNodes(D)
%
% find terminal nodes (do not have leaves) of pruned tree structure
% find lowest approximation node
%
% D   node data structure
% tn  vector of terminal nodes
% an  index of lowest approximation node and index of node with higest node energy

% G. Doblinger, TU-Wien, 12.9.97

Nnodes = length(D.entr);   % number of nodes in the tree
N = log2(Nnodes+2)-1;      % number of tree levels

% search first N-1 tree levels for terminal nodes

tn = zeros(1,Nnodes);
n1 = 2^N-2;
for n = 1:n1
   if ~isnan(D.entr(n)) && isnan(D.entr(2*n+1))
      tn(n) = n;           % terminal node if entropy not NaN but leaf is NaN
   end
end

% search nodes of last level N

for n = n1+1:Nnodes
   if ~isnan(D.entr(n))
      tn(n) = n;
   end
end

tn = find(tn > 0);

% find approximation node
% approximation nodes can be nodes with indices 1,3,7,15,31,...

for k = 1:N
   n = 2^k-1;
   l = find(tn == n);
   if ~isempty(l)
      an = tn(l);
      break
   end
end

% add node with maximum energy to nodes not to be thresholded

[~,an1] = max(D.engy(tn));
an = [an tn(an1)];
