function D = WaveletTree(D)

% extract wavelet tree from full binary tree
% D            structure of node data
%              D.coef     node coefficients concatenated to row vector
%              D.start    start index of coeff. for each node
%              D.stop     stop index of coeff. for each node
%              D.root     index of root node for each node (top node = index 0)
%              D.entr     entropy for each node
%              D.engy     energy for each node

% G. Doblinger, TU-Wien, 25.12.97

% nodes to be eliminated are marked with entropy D.entr = NaN
% Note: topmost node (index 0) is not used)

Nnodes = length(D.entr);   % number of nodes in the tree
if Nnodes == 1
   return
end
N = log2(Nnodes+2)-1;      % number of tree levels

Etemp = D.entr;
D.entr(1:Nnodes) = NaN;    % mark all nodes with NaNs
k = 2.^(1:N);
D.entr(k) = Etemp(k);      % restore wavelet nodes
D.entr(k-1) = Etemp(k-1);
