% multiresolution demo using wavelets
%
% G. Doblinger, TU-Wien, 11-2009

% compute wavelet decomposition and reconstruction filters

%wname = [];              % do not use MATLAB toolbox function
%wname = 'db1';          % use MATLAB toolbox
wname = 'coif4';          % use MATLAB toolbox

if ~isempty(wname)
   [LoF_D,HiF_D,LoF_R,HiF_R] = wfilters(wname);
else
   h = qmf_2(44,0.7,0.1);   
   Lh = length(h)/4;
   LoF_D = h(1:Lh);
   HiF_D = h(Lh+1:2*Lh);
   LoF_R = h(2*Lh+1:3*Lh);
   HiF_R = h(3*Lh+1:4*Lh);
end

%%

% select type of test signal

sigtype = 'blocks';
%sigtype = 'bumps';      
%sigtype = 'doppler';   

Nx = 2000;
x = signals(sigtype,Nx);

% wavelet decomposition and signal reconstruction 

Nlevels = 5;              % levels of wavelet tree (determines resolution
                          % steps)                

% compute node coefficients of full binary tree and wavelet tree
%
% Node labeling of full binary tree (3 levels)
%           ___ 0 ___
%          /         \
%         1           2
%       /   \       /   \
%      3     4     5     6
%     / \   / \   / \   / \
%    7   8 9  10 11 12 13 14 
%
% terminal nodes are nodes without leafs
% e.g. wavelet tree: 2, 4, 7, 8
% with approximation node 7, and detail nodes 2, 4, 8

D = CompleteTree(x,LoF_D,HiF_D,Nlevels);  % binary tree
D = WaveletTree(D);       % wavelet tree
[tn,an] = TermNodes(D);   % get terminal nodes and approximation node
tn = setdiff(tn,an);      % detail nodes (terminal nodes without an)
Ntn = length(tn);

% set selected detail node coefficients to zero to change resolution
%
% wavelet tree node labels: 7 = approximation node, 2, 4, 8 are detail nodes
%          0 = original signal
%         / \
%        1   2 = tn(1)
%       / \ 
%      3   4 = tn(2)
%     / \ 
%    7   8 = tn(3)
%   an 
%   an = signal approximation without details

% the more details the sharper the signal view (similar to photography)

res = 0;                  % res = 0 means no details, only approximaton node
                          % res = Ntn means all details = full resolution

tn1 = tn(1:Ntn-res);      % terminal nodes which are not used 
for n = tn1
   k = D.start(n):D.stop(n);
   D.coef(k) = 0;
end

% signal reconstruction with selected terminal nodes

y = WavePackRec(D,LoF_R,HiF_R);

close all;
figure('name','time domain','Units','normal',...
   'Position',[0.5,0.5,0.49,0.45]);

n = 0:Nx-1;
subplot(2,1,1), plot(n,x);
xlabel('n'), ylabel('x[n]');
title('original signal');
subplot(2,1,2), plot(n,y);
xlabel('n'), ylabel('y[n]');
title(sprintf('reconstructed signal, resolution = %d',res));

