function x = signals(sigtype,Nx)
% function x = signals(sigtype,Nx)
%
% create test signals for wavelet transform
%
% sigtype = 'blocks' or 'bumps' or 'doppler'
% Nx        signal length
%
% functions from D. Donoho and I. Johnstone, "Adapting to unknown
% smoothness via wavelet shrinkage", Preprint Stanford, January 93, p 27-28.

t = [0.1  0.13  0.15  0.23  0.25  0.40  0.44  0.65  0.76  0.78  0.81]';
h = [4      -5     3    -4     5  -4.2   2.1   4.3  -3.1   5.1  -4.2]';

x = zeros(Nx,1);
tt = linspace(0,1,Nx)';
Nt = length(t);
switch lower(sigtype)   
   case 'blocks'
      for n=1:Nt
         x = x + h(n)*(1+sign(tt-t(n)))/2;
      end
   case 'bumps'
      h  = abs(h);     
      w  = 0.01*[0.5 0.5 0.6 1 1 3 1 1 0.5 0.8 0.5];
      for n=1:Nt
         x = x + h(n)./(1+((tt-t(n))/w(n)).^4);
      end
   case 'doppler'
      x = linspace(0,1,Nx)';
      epsi = 0.05;
      x = sqrt(x.*(1-x)).*sin((2*pi+epsi)./(x+epsi));
  otherwise
      error('signal type not supported');
end

